//============================================================================//
//
//  CAPBONUS
//
//    author:   Dave Rodgers  (a.k.a. trepan)
//    brief:    BZFS plugin to add bonus points during team flag captures
//    date:     June 28, 2010
//    license:  LGPL v2.1
//
//  NOTE: this could easily be adjusted to dole out bonuses based on the
//        number of enemies player currently on the board, or variations
//        of the same.
//
//============================================================================//

#include "bzfs.h"
#include "bzfsAPI.h"
#include "plugin_utils.h"

#include <string>
#include <ctype.h>

#include "incscore.h"


BZ_GET_PLUGIN_VERSION


static const std::string dummyName = "capbonus";

static const std::string defaultCapBonus = "5,1"; // player,team


//============================================================================//

static inline GameKeeper::Player* getPlayerByIndex(int playerID) {
  return GameKeeper::Player::getPlayerByIndex(playerID);
}

//============================================================================//

static int ParseBonusValue(const char* s, int /*allies*/, int /*enemies*/) {
  char* end;
  const int numVal = strtol(s, &end, 0);
  if (end == s) {
    return -1;
  }
  return numVal;
}


static const char* GetTeamName(TeamColor t) {
  switch (t) {
    case RedTeam:    { return "Red";    }
    case GreenTeam:  { return "Green";  }
    case BlueTeam:   { return "Blue";   }
    case PurpleTeam: { return "Purple"; }
    default:         { return NULL;     }
  }
}


static int GetTeamCount(const TeamColor t) {
  int count = 0;
  for (int i = 0; i < curMaxPlayers; i++) {
    GameKeeper::Player *p = GameKeeper::Player::getPlayerByIndex(i);
    if (p && p->player.isPlaying() && p->player.isTeam(t)) {
      count++;
    }
  }
  return count;
}


//============================================================================//

class CapHandler : public bz_EventHandler {
  public:
    void process(bz_EventData* ed);
};


static CapHandler capHandler;


void CapHandler::process(bz_EventData* ed) {
  const bz_CTFCaptureEventData* e = (const bz_CTFCaptureEventData*) ed;

  GameKeeper::Player* capper = getPlayerByIndex(e->playerCapping);
  if (capper == NULL) {
    return;
  }

  const TeamColor capTeam = capper->player.getTeam();
  const TeamColor vicTeam = (TeamColor)convertTeam(e->teamCapped);

  if (capTeam == vicTeam) {
    return;
  }

  const int allies  = GetTeamCount(capTeam);
  const int enemies = GetTeamCount(vicTeam);

  std::string capBonus = BZDB.get("_capBonus");
  if (capBonus.empty()) {
    capBonus = defaultCapBonus;
  }

  std::string playerStr, teamStr;
  const char* s = capBonus.c_str();
  const char* comma = strstr(s, ",");
  if (comma == NULL) {
    playerStr = capBonus;
  } else {
    playerStr = std::string(s, comma - s);
    teamStr = (comma + 1);
  }

  const int playerBonus = ParseBonusValue(playerStr.c_str(), allies, enemies);
  const int teamBonus   = ParseBonusValue(teamStr.c_str(),   allies, enemies);

  // player bonus
  if (playerBonus > 0) {
    IncrementPlayerScore(e->playerCapping, playerBonus, dummyName);

    // public message
    char msgBuf[256];
    snprintf(msgBuf, sizeof(msgBuf),
             "capbonus: %s awarded %i bonus point%s.",
             capper->player.getCallSign(), playerBonus,
             playerBonus > 1 ? "s" : "");
    bz_sendTextMessage(BZ_SERVER, BZ_ALLUSERS, msgBuf);
  }

  // team bonus
  if (teamBonus > 0) {
    const char* teamName = GetTeamName(capTeam);
    if (teamName != NULL) {
      int hits = 0;
      for (int pi = 0; pi < curMaxPlayers; pi++) {
        if (pi == e->playerCapping) {
          continue; // note the exclusion
        }
        GameKeeper::Player* p = getPlayerByIndex(pi);
        if (p && p->player.isPlaying() && p->player.isTeam(capTeam)) {
          IncrementPlayerScore(pi, teamBonus, dummyName);
          hits++;
        }
      }
      // public message
      if (hits > 0) {
        char msgBuf[256];
        snprintf(msgBuf, sizeof(msgBuf),
                 "capbonus: %s team members awarded %i bonus point%s.",
                 teamName, teamBonus,
                 teamBonus > 1 ? "s" : "");
        bz_sendTextMessage(BZ_SERVER, BZ_ALLUSERS, msgBuf);
      }
    }
  }
}


//============================================================================//
//
//  [[ /fakecap capper team ]] 
//

class FakecapHandler : public bz_CustomSlashCommandHandler {
  public:
    bool handle(int playerID, bzApiString command,
                bzApiString message,
                bzAPIStringList *params);
};


static FakecapHandler fakecapHandler;


static bz_eTeamType ParseTeam(const std::string& str) {
  const std::string lower = tolower(str);
  if (lower == "rogue")  { return eRogueTeam;  }
  if (lower == "red")    { return eRedTeam;    }
  if (lower == "green")  { return eGreenTeam;  }
  if (lower == "blue")   { return eBlueTeam;   }
  if (lower == "purple") { return ePurpleTeam; }
  return eNoTeam;
}


// "/fakecap capper team"
bool FakecapHandler::handle(int playerID, bzApiString command,
                            bzApiString /*message*/,
                            bzAPIStringList *params) {
  if (command != "fakecap") {
    return false;
  }

  GameKeeper::Player* gkPlayer = getPlayerByIndex(playerID);
  if (gkPlayer == NULL) {
    return true;
  }
  if (!gkPlayer->accessInfo.isOperator()) {
    bz_sendTextMessage(BZ_SERVER, playerID, "fakecap: you are not an operator");
    return true;
  }

  if (params->size() != 2) {
    bz_sendTextMessage(BZ_SERVER, playerID, "usage: /fakecap <player> <team>");
    return true;
  }
  const std::string playerStr = params->get(0).c_str();
  const std::string teamStr   = params->get(1).c_str();

  const int capperID = GameKeeper::Player::getPlayerIDByName(playerStr.c_str());
  GameKeeper::Player* capper = getPlayerByIndex(capperID);
  if (capper == NULL) {
    bz_sendTextMessage(BZ_SERVER, playerID, "fakecap: invalid player");
    return true;
  }

  const bz_eTeamType vicTeam = ParseTeam(teamStr);
  if (vicTeam == eNoTeam) {
    bz_sendTextMessage(BZ_SERVER, playerID, "fakecap: invalid team");
    return true;
  }

  bz_sendTextMessage(BZ_SERVER, BZ_ALLUSERS, "FAKE CAPTURE!!!");

  bz_CTFCaptureEventData ce;
  ce.teamCapped    = vicTeam;
  ce.teamCapping   = convertTeam(capper->player.getTeam());
  ce.playerCapping = capperID;
  capHandler.process(&ce);

  return true;
}


//============================================================================//

BZF_PLUGIN_CALL int bz_Load(const char* /*commandLine*/) {
  bz_debugMessage(4, "capbonus plugin loaded");
  bz_registerEvent(bz_eCaptureEvent, &capHandler);
  bz_registerCustomSlashCommand("fakecap", &fakecapHandler);
  return 0;
}


BZF_PLUGIN_CALL int bz_Unload() {
  bz_debugMessage(4, "capbonus plugin unloaded");
  bz_removeEvent(bz_eCaptureEvent, &capHandler);
  bz_removeCustomSlashCommand("fakecap");
  return 0;
}


//============================================================================//


// Local Variables: ***
// mode:C++ ***
// tab-width: 8 ***
// c-basic-offset: 2 ***
// indent-tabs-mode: t ***
// End: ***
// ex: shiftwidth=2 tabstop=8

